using System;
using System.Globalization;
using System.Reflection;
using System.Web;
using Microsoft.Office.Interop.Word;
using Telerik.Web.UI.Spell;

namespace Telerik.Web.UI
{
	/// <summary>
	/// MS Word Spell Checking Provider for RadSpell (ASP.NET AJAX version)
	/// </summary>
	public class WordSpellCheckProvider : ISpellCheckProvider
	{
		private WordIgnoreOptions _wordIgnoreOptions;
		private ApplicationClass application = new ApplicationClass();
		private Document document;
		private string language = "en-US";
		private object missing = Missing.Value;

		public WordSpellCheckProvider()
		{
			this.application.Visible = false;
			this.CreateDocument();
		}

		public WordSpellCheckProvider(Telerik.Web.UI.SpellChecker checker)
			: this()
		{
			if (checker.DictionaryLanguage != string.Empty)
			{
				this.Language = checker.DictionaryLanguage;
			}
			this.WordIgnoreOptions = checker.WordIgnoreOptions;
			this.Text = checker.Text;
		}

		public bool CheckWord(ITextWord current, ITextWord previous)
		{
			if (this.CheckForRepeatWords && previous != null && (current.Word == previous.Word))
			{
				return false;
			}
			if (!this.CheckCapital && current.StartsWithUpper())
			{
				return true;
			}
			MicrosoftTextWord word = null;
			foreach (Range range in this.document.Words)
			{
				if (range.Text.Trim() == current.Word)
				{
					word = new MicrosoftTextWord(range, this.language);
					break;
				}
			}
			return (word.Container.SpellingErrors.Count == 0);
		}

		public void Close()
		{
			if (this.application != null)
			{
				this.document = null;
				object saveChanges = null;
				this.application.Quit(ref saveChanges, ref saveChanges, ref saveChanges);
				this.application = null;
			}
		}

		private void CreateDocument()
		{
			Document document = this.application.Documents.Add(ref this.missing, ref this.missing, ref this.missing, ref this.missing);
			document.LanguageDetected = true;
			this.document = document;
		}

		~WordSpellCheckProvider()
		{
			this.Close();
		}

		private SpellingSuggestions GetAppSuggestions(ITextWord word)
		{
			SpellingSuggestions suggestionsForRange = null;
			MicrosoftTextWord word2 = null;
			foreach (Range range in this.document.Words)
			{
				if (range.Text.Trim() == word.Word)
				{
					word2 = new MicrosoftTextWord(range, this.language);
					break;
				}
			}
			if (word2 != null)
			{
				suggestionsForRange = this.GetSuggestionsForRange(word2);
			}
			return suggestionsForRange;
		}

		private string GetDocumentText()
		{
			if (this.document == null)
			{
				return string.Empty;
			}
			return this.document.Content.Text;
		}

		public string[] GetSuggestions(ITextWord word)
		{
			SpellingSuggestions appSuggestions = this.GetAppSuggestions(word);
			if ((appSuggestions == null) || (appSuggestions.SpellingErrorType == WdSpellingErrorType.wdSpellingCorrect))
			{
				return new string[0];
			}
			string[] strArray = new string[appSuggestions.Count];
			for (int i = 0; i < appSuggestions.Count; i++)
			{
				SpellingSuggestion suggestion = appSuggestions[i + 1];
				strArray[i] = suggestion.Name;
			}
			return strArray;
		}

		private SpellingSuggestions GetSuggestionsForRange(MicrosoftTextWord word)
		{
			Range container = word.Container;
			object ignoreUppercase = !this.CheckAllCaps;
			return container.GetSpellingSuggestions(ref this.missing, ref ignoreUppercase, ref this.missing, ref this.missing, ref this.missing, ref this.missing, ref this.missing, ref this.missing, ref this.missing, ref this.missing, ref this.missing, ref this.missing, ref this.missing);
		}

		private SpellingSuggestions GetSuggestionsForString(string word)
		{
			object ignoreUppercase = !this.CheckAllCaps;
			return this.application.GetSpellingSuggestions(word, ref this.missing, ref ignoreUppercase, ref this.missing, ref this.missing, ref this.missing, ref this.missing, ref this.missing, ref this.missing, ref this.missing, ref this.missing, ref this.missing, ref this.missing, ref this.missing);
		}

		public ITextWord GetWord(int index)
		{
			return new MicrosoftTextWord(this.document.Words[index + 1], this.language);
		}

		private void SetDocumentText(string text)
		{
			this.CreateDocument();
			Range content = this.document.Content;
			content.Text = text;
			content.LanguageID = MicrosoftTextWord.GetLanguage(this.Language);
		}

		// Properties
		private bool CheckAllCaps
		{
			get
			{
				return ((this.WordIgnoreOptions & Telerik.Web.UI.WordIgnoreOptions.UPPERCASE) == 0);
			}
		}

		private bool CheckCapital
		{
			get
			{
				return ((this.WordIgnoreOptions & Telerik.Web.UI.WordIgnoreOptions.WordsWithCapitalLetters) == 0);
			}
		}

		private bool CheckForRepeatWords
		{
			get
			{
				return ((this.WordIgnoreOptions & Telerik.Web.UI.WordIgnoreOptions.RepeatedWords) == 0);
			}
		}

		private bool CheckWordsWNumbers
		{
			get
			{
				return ((this.WordIgnoreOptions & Telerik.Web.UI.WordIgnoreOptions.WordsWithNumbers) == 0);
			}
		}

		[CLSCompliant(false)]
		public Document CurrentDocument
		{
			get
			{
				return this.document;
			}
		}

		public string Language
		{
			get
			{
				return this.language;
			}
			set
			{
				this.language = value;
			}
		}

		public string Text
		{
			get
			{
				return this.GetDocumentText();
			}
			set
			{
				this.SetDocumentText(value);
			}
		}

		public int WordCount
		{
			get
			{
				return this.document.Words.Count;
			}
		}

		public WordIgnoreOptions WordIgnoreOptions
		{
			get
			{
				return this._wordIgnoreOptions;
			}
			set
			{
				this._wordIgnoreOptions = value;
			}
		}
	}
	internal class MicrosoftTextWord : ITextWord
	{
		private Range container;
		private string language;

		internal MicrosoftTextWord(Range container, string languageName)
		{
			this.container = container;
			this.language = languageName;
			this.container.LanguageID = GetLanguage(this.language);
		}

		public bool AllUpper()
		{
			string text = this.container.Text;
			return (text.ToUpper() == text);
		}

		internal static WdLanguageID GetLanguage(string languageName)
		{
			CultureInfo info = new CultureInfo(languageName);
			return (WdLanguageID)Enum.Parse(typeof(WdLanguageID), info.LCID.ToString(), false);
		}

		public void MakeUpper()
		{
			this.container.Text = this.container.Text.ToUpper();
		}

		public bool StartsWithUpper()
		{
			string text = this.container.Text;
			return ((text.ToUpper()[0] == text[0]) && char.IsLetter(text[0]));
		}

		// Properties
		internal Range Container
		{
			get
			{
				return this.container;
			}
		}

		public string HtmlWord
		{
			get
			{
				return HttpUtility.HtmlEncode(this.Word);
			}
		}

		public int Offset
		{
			get
			{
				return this.container.Start;
			}
		}

		public string Word
		{
			get
			{
				return this.container.Text.Trim();
			}
		}
	}
}
